/* 
 * VirtualBox Control Service
 * 
 * Copyright © 2010 vu1tur.eu.org
 * 
 * This file is part of VirtualBox Control Service
 * 
 * VirtualBox Control Service is free software; you can redistribute it 
 * and/or modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2 of 
 * the License, or (at your option) any later version.
 * 
 * VirtualBox Control Service is distributed in the hope that it will 
 * be useful, but WITHOUT ANY WARRANTY; without even the implied 
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

#include "vboxctrl.h"
#include "vboxipc.h"
#include <stdio.h>

BOOL fIPCListenerRunning;

void VBoxIPCListener(void* args)
{
	PSECURITY_DESCRIPTOR psd;
	PACL pacl;
	EXPLICIT_ACCESS ea[2];
	SECURITY_ATTRIBUTES sa;

	BuildExplicitAccessWithName(ea, TEXT("INTERACTIVE"), ACCESS_SYSTEM_SECURITY | FILE_GENERIC_READ | FILE_GENERIC_WRITE, GRANT_ACCESS, NO_INHERITANCE);
	BuildExplicitAccessWithName(ea+1, TEXT("SYSTEM"), ACCESS_SYSTEM_SECURITY | FILE_GENERIC_READ | FILE_GENERIC_WRITE, GRANT_ACCESS, NO_INHERITANCE);
	SetEntriesInAcl(2, ea, NULL, &pacl);
	psd = (PSECURITY_DESCRIPTOR)LocalAlloc(LPTR, SECURITY_DESCRIPTOR_MIN_LENGTH);
	InitializeSecurityDescriptor(psd, SECURITY_DESCRIPTOR_REVISION);
	SetSecurityDescriptorDacl(psd, TRUE, pacl, FALSE);
	sa.nLength = sizeof(SECURITY_ATTRIBUTES);
	sa.lpSecurityDescriptor = psd;
	sa.bInheritHandle = FALSE;
	
	char szPipe[256];
	sprintf(szPipe, "%s.%s", VBOX_PIPENAME_BASE, szvmName);
	
	HANDLE hPipe = INVALID_HANDLE_VALUE;

	fIPCListenerRunning = TRUE;
	while (fIPCListenerRunning)
	{
		hPipe = CreateNamedPipe(
			szPipe,
			PIPE_ACCESS_DUPLEX,	
			PIPE_TYPE_BYTE | PIPE_READMODE_BYTE | PIPE_WAIT,
			PIPE_UNLIMITED_INSTANCES,
			16,
			16,
			NMPWAIT_USE_DEFAULT_WAIT,
			&sa);
		
		if (hPipe == INVALID_HANDLE_VALUE) {
			Log(1, "Error creating pipe (0x%08x)", GetLastError());
			break;
		}

		BOOL fConnected = ConnectNamedPipe(hPipe, NULL) ? TRUE : GetLastError() == ERROR_PIPE_CONNECTED;
	
		if (fConnected)
			_beginthread(VBoxIPCDispatcher, 0, (void*)hPipe);
		else
			CloseHandle(hPipe);
	}
	LocalFree(pacl);
	LocalFree(psd);
}

void VBoxIPCDispatcher(HANDLE hp)
{
	if (hp != INVALID_HANDLE_VALUE)
	{
		DWORD dwBuffer;
		DWORD cbBytes;
			
		BOOL bResult = ReadFile(hp, (void*)&dwBuffer, sizeof(dwBuffer), &cbBytes, NULL);
		if (!bResult || (cbBytes == 0))
		{
			Log(1, "ERROR reading from client");
		} else {
			switch (dwBuffer)
			{
				case VBOX_IPC_C_GETVRDPENABLED:
					VBoxIPC_GetVRDPEnabled(hp);
					break;
				case VBOX_IPC_C_GETVRDPPORT:
					VBoxIPC_GetVRDPPort(hp);
					break;
				case VBOX_IPC_C_GETVRDPRESOLUTION:
					VBoxIPC_GetVRDPResolution(hp);
					break;
				default:
					break;
			}
		}
		FlushFileBuffers(hp);
		DisconnectNamedPipe(hp);
		CloseHandle(hp);
	}
}

void VBoxIPCShutdown()
{
	char szPipe[256];
	fIPCListenerRunning = FALSE;
	sprintf(szPipe, "%s.%s", VBOX_PIPENAME_BASE, szvmName);
	HANDLE hPipe = CreateFile(szPipe, GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, NULL);
	DWORD dwFunc = 0, cbBytes;
	WriteFile(hPipe, &dwFunc, sizeof(dwFunc), &cbBytes, NULL);
	CloseHandle(hPipe);
}

void VBoxIPC_GetVRDPEnabled(HANDLE hp)
{
	IVirtualBox *tmpVirtualBox;
	ISession *tmpSession;
	InitializeVirtualBoxInterface(&tmpVirtualBox);
	OpenVirtualBoxSession(&tmpSession);
	DWORD cbBytes;
	DWORD enabled = (DWORD) GetMachineVRDPEnabled(tmpVirtualBox, tmpSession);
	WriteFile(hp, &enabled, sizeof(enabled), &cbBytes, NULL);
	SAFE_RELEASE(tmpSession);
	CloseVirtualBoxInterface(&tmpVirtualBox);
}

void VBoxIPC_GetVRDPPort(HANDLE hp)
{
	IVirtualBox *tmpVirtualBox;
	ISession *tmpSession;
	InitializeVirtualBoxInterface(&tmpVirtualBox);
	OpenVirtualBoxSession(&tmpSession);
	DWORD cbBytes;
	DWORD port = (DWORD) GetMachineVRDPPort(tmpVirtualBox, tmpSession);
	WriteFile(hp, &port, sizeof(port), &cbBytes, NULL);
	SAFE_RELEASE(tmpSession);
	CloseVirtualBoxInterface(&tmpVirtualBox);	
}

void VBoxIPC_GetVRDPResolution(HANDLE hp)
{
	IVirtualBox *tmpVirtualBox;
	ISession *tmpSession;
	InitializeVirtualBoxInterface(&tmpVirtualBox);
	OpenVirtualBoxSession(&tmpSession);
	DWORD cbBytes;
	DWORD resolution = (DWORD) GetMachineVRDPResolution(tmpVirtualBox, tmpSession);
	WriteFile(hp, &resolution, sizeof(resolution), &cbBytes, NULL);
	SAFE_RELEASE(tmpSession);
	CloseVirtualBoxInterface(&tmpVirtualBox);
}
