/* 
 * VirtualBox Control Service
 * 
 * Copyright © 2010 vu1tur.eu.org
 * 
 * This file is part of VirtualBox Control Service
 * 
 * VirtualBox Control Service is free software; you can redistribute it 
 * and/or modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2 of 
 * the License, or (at your option) any later version.
 * 
 * VirtualBox Control Service is distributed in the hope that it will 
 * be useful, but WITHOUT ANY WARRANTY; without even the implied 
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */
 
#include "utils.h"

typedef DWORD ARGB;


void strcrpl(char *str, char c, char r)
{
	unsigned int i;
	if (str == NULL)
		return;
	for (i = 0; i < strlen(str); i++)
	{
		if (str[i] == c)
			str[i] = r;
	}
}

void InitBitmapInfo(BITMAPINFO  *pbmi, ULONG cbInfo, LONG cx, LONG cy)
{
	ZeroMemory(pbmi, cbInfo);
	pbmi->bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
	pbmi->bmiHeader.biPlanes = 1;
	pbmi->bmiHeader.biCompression = BI_RGB;
	pbmi->bmiHeader.biWidth = cx;
	pbmi->bmiHeader.biHeight = cy;
	pbmi->bmiHeader.biBitCount = 32;
}

HRESULT Create32BitHBITMAP(HDC hdc, void **ppvBits, HBITMAP* phBmp)
{
	*phBmp = NULL;
	BITMAPINFO bmi;
	InitBitmapInfo(&bmi, sizeof(bmi), 16, 16);
	HDC hdcUsed = hdc ? hdc : GetDC(NULL);
	if (hdcUsed)
	{
		*phBmp = CreateDIBSection(hdcUsed, &bmi, DIB_RGB_COLORS, ppvBits, NULL, 0);
		if (hdc != hdcUsed)
			ReleaseDC(NULL, hdcUsed);
	}
	return (NULL == *phBmp) ? E_OUTOFMEMORY : S_OK;
}

HRESULT ConvertToPARGB32(HDC hdc, ARGB *pargb, HBITMAP hbmp, int cxRow)
{
	BITMAPINFO bmi;
	InitBitmapInfo(&bmi, sizeof(bmi), 16, 16);
	HRESULT hr = E_OUTOFMEMORY;
	HANDLE hHeap = GetProcessHeap();
	void *pvBits = HeapAlloc(hHeap, 0, bmi.bmiHeader.biWidth * 4 * bmi.bmiHeader.biHeight);
	if (pvBits)
	{
		hr = E_UNEXPECTED;
		if (GetDIBits(hdc, hbmp, 0, bmi.bmiHeader.biHeight, pvBits, &bmi, DIB_RGB_COLORS) == bmi.bmiHeader.biHeight)
		{
			ULONG cxDelta = cxRow - bmi.bmiHeader.biWidth;
			ARGB *pargbMask = (ARGB *)pvBits;
			ULONG x,y;
			for (y = bmi.bmiHeader.biHeight; y; --y)
			{
				for (x = bmi.bmiHeader.biWidth; x; --x)
				{
					if (*pargbMask++)
						*pargb++ = 0x00ffffff;
					else
						*pargb++ |= 0xff000000;
				}
				pargb += cxDelta;
			}
			hr = S_OK;
		}
		HeapFree(hHeap, 0, pvBits);
	}
	return hr;
}

HasAlpha(ARGB *pargb, int cxRow)
{
	ULONG cxDelta = cxRow - 16;
	ULONG x, y;
	for (y = 16;y;--y)
	{
		for (x = 16;x;--x)
			if (*pargb++ & 0xFF000000)
				return TRUE;
		pargb += cxDelta;
	}
	return FALSE;
}

HRESULT ConvertBufferToPARGB32(HPAINTBUFFER hPaintBuffer, HDC hdc, HICON hicon)
{
	RGBQUAD *prgbQuad;
	int cxRow;
	HRESULT hr = getBufferedPaintBits(hPaintBuffer, &prgbQuad, &cxRow);
	if (SUCCEEDED(hr))
	{
		ARGB *pargb = (ARGB *)prgbQuad;
		if (!HasAlpha(pargb, cxRow))
		{
			ICONINFO info;
			if (GetIconInfo(hicon, &info))
			{
				if (info.hbmMask)
					hr = ConvertToPARGB32(hdc, pargb, info.hbmMask, cxRow);
				
				DeleteObject(info.hbmColor);
				DeleteObject(info.hbmMask);
			}
		}
	}
	return hr;
}

HBITMAP IconToBitmap(HICON hicon)
{
	HRESULT hr;
	HBITMAP hbmp = NULL;
	RECT rcIcon;
	SetRect(&rcIcon, 0, 0, 16, 16);
	HDC hdcDest = CreateCompatibleDC(NULL);
	if (hdcDest)
	{
		hr = Create32BitHBITMAP(hdcDest, NULL, &hbmp);
		if (SUCCEEDED(hr))
		{
			hr = E_FAIL;
			HBITMAP hbmpOld = (HBITMAP)SelectObject(hdcDest, hbmp);
			if (hbmpOld)
			{
				BLENDFUNCTION bfAlpha = { AC_SRC_OVER, 0, 255, AC_SRC_ALPHA };
				BP_PAINTPARAMS paintParams = { 0 };
				paintParams.cbSize = sizeof(paintParams);
				paintParams.dwFlags = BPPF_ERASE;
				paintParams.pBlendFunction = &bfAlpha;
				
				HDC hdcBuffer;
				HPAINTBUFFER hPaintBuffer = beginBufferedPaint(hdcDest, &rcIcon, BPBF_DIB, &paintParams, &hdcBuffer);
				if (hPaintBuffer)
				{
					if (DrawIconEx(hdcBuffer, 0, 0, hicon, 16, 16, 0, NULL, DI_NORMAL))
						hr = ConvertBufferToPARGB32(hPaintBuffer, hdcDest, hicon);
					endBufferedPaint(hPaintBuffer, TRUE);
				}
				SelectObject(hdcDest, hbmpOld);
			}
		}
		DeleteDC(hdcDest);
	}

	return hbmp;
}
