/* 
 * VirtualBox Control Service
 * 
 * Copyright © 2010 vu1tur.eu.org
 * 
 * This file is part of VirtualBox Control Service
 * 
 * VirtualBox Control Service is free software; you can redistribute it 
 * and/or modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2 of 
 * the License, or (at your option) any later version.
 * 
 * VirtualBox Control Service is distributed in the hope that it will 
 * be useful, but WITHOUT ANY WARRANTY; without even the implied 
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */
 
#include "vboxipcc.h"
#include <stdio.h>
 
BOOL VBoxIPCC_GetVRDPEnabled(const char *vmname)
{
	char pipename[256];
	if (strlen(vmname) + strlen(VBOX_PIPENAME_BASE) > 255)
		return FALSE;
	sprintf(pipename, "%s.%s", VBOX_PIPENAME_BASE, vmname);

	DWORD cbBytes;
	HANDLE hPipe = CreateFile(pipename, GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, NULL);
	if (hPipe == INVALID_HANDLE_VALUE)
		return FALSE;
	DWORD enabled = 0;
	DWORD dwFunc = VBOX_IPC_C_GETVRDPENABLED;
	BOOL bResult = WriteFile(hPipe, &dwFunc, sizeof(dwFunc), &cbBytes, NULL);
	if (bResult)
		ReadFile(hPipe, &enabled, sizeof(enabled), &cbBytes, NULL);
	CloseHandle(hPipe);
	
	return (BOOL)enabled;
}
 
LONG VBoxIPCC_GetVRDPPort(const char *vmname)
{
	char pipename[256];
	if (strlen(vmname) + strlen(VBOX_PIPENAME_BASE) > 255)
		return FALSE;
	sprintf(pipename, "%s.%s", VBOX_PIPENAME_BASE, vmname);
	
	DWORD cbBytes;
	HANDLE hPipe = CreateFile(pipename, GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, NULL);
	if (hPipe == INVALID_HANDLE_VALUE)
		return -1;
	DWORD port;
	DWORD dwFunc = VBOX_IPC_C_GETVRDPPORT;
	BOOL bResult = WriteFile(hPipe, &dwFunc, sizeof(dwFunc), &cbBytes, NULL);
	if (bResult)
		ReadFile(hPipe, &port, sizeof(port), &cbBytes, NULL);
	CloseHandle(hPipe);
	
	return (LONG)port;
}
 
BOOL VBoxIPCC_GetVRDPResolution(const char *vmname, LONG *width, LONG *height)
{
	 char pipename[256];
	 if (strlen(vmname) + strlen(VBOX_PIPENAME_BASE) > 255)
		 return FALSE;
	sprintf(pipename, "%s.%s", VBOX_PIPENAME_BASE, vmname);
	
	DWORD cbBytes;
	HANDLE hPipe = CreateFile(pipename, GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, NULL);
	if (hPipe == INVALID_HANDLE_VALUE)
		return -1;
	DWORD resolution;
	DWORD dwFunc = VBOX_IPC_C_GETVRDPRESOLUTION;
	BOOL bResult = WriteFile(hPipe, &dwFunc, sizeof(dwFunc), &cbBytes, NULL);
	if (bResult)
		ReadFile(hPipe, &resolution, sizeof(resolution), &cbBytes, NULL);
	CloseHandle(hPipe);
	*width = LOWORD(resolution);
	*height = HIWORD(resolution);
	
	return TRUE;
}